<?php

namespace Drupal\simple_sitemap_language_hierarchy\Plugin\simple_sitemap\UrlGenerator;

use Drupal\simple_sitemap\Plugin\simple_sitemap\UrlGenerator\EntityUrlGenerator;
use Drupal\Core\Url;
use Drupal\Core\Entity\ContentEntityInterface;

/**
 * Handle all url generate based on language hierarchy.
 *
 * @package Drupal\simple_sitemap_language_hierarchy\Plugin\simple_sitemap\UrlGenerator
 *
 * @UrlGenerator(
 *   id = "language_hierarchy_url_generator",
 *   label = @Translation("Language Hierarchy Entity URL generator"),
 *   description = @Translation("Generates URLs for entity bundles and bundle overrides."),
 * )
 */
class LanguageHierarchyUrlGenerator extends EntityUrlGenerator {

  /**
   * Gets the alternate URLs for translated languages.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to process.
   * @param \Drupal\Core\Url $url_object
   *   The URL object.
   *
   * @return array
   *   An array of alternate URLs.
   */
  protected function getAlternateUrlsForTranslatedLanguages(ContentEntityInterface $entity, Url $url_object): array {
    $alternate_urls = [];

    // Update the function outcome to give the same result as the service.
    $hierarchCache = \Drupal::service('simple_sitemap_language_hierarchy.manager');
    $hierarchyArray = $hierarchCache->getCachedHierarchyArray();
    $excludeLanguages = $this->settings->get('excluded_languages');

    foreach ($entity->getTranslationLanguages() as $language) {
      $langCode = $language->getId();

      if ($entity->getTranslation($language->getId())->access('view', $this->anonUser)) {
        if (!isset($this->settings->get('excluded_languages')[$language->getId()]) && !$language->isDefault()) {
          $alternate_urls[$language->getId()] = $this->replaceBaseUrlWithCustom($url_object
            ->setAbsolute()->setOption('language', $language)->toString());
        }

        // Get hierarchy form translated languages.
        foreach (array_keys($hierarchyArray[$langCode]['#children']) as $childLanguage) {
          $languageObj = $this->languages[$childLanguage];
          if (!isset($excludeLanguages[$childLanguage]) && !$languageObj->isDefault()) {
            $alternate_urls[$childLanguage] = $this->replaceBaseUrlWithCustom($url_object
              ->setAbsolute()->setOption('language', $languageObj)->toString());
          }
        }
      }
    }
    return $alternate_urls;

  }

}
